/*
 * (C) Copyright 2016 Nuxeo SA (http://nuxeo.com/) and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package org.nuxeo.ecm.platform.importer.mqueues.pattern.consumer;

import org.nuxeo.ecm.platform.importer.mqueues.pattern.Message;


/**
 * A consumer consumes {@link Message} by batch.
 *
 * @since 9.1
 */
public interface Consumer<M extends Message> extends AutoCloseable {

    /**
     * Begin a batch of messages.
     */
    void begin();

    /**
     * Consume a message.
     */
    void accept(M message);

    /**
     * End of the batch message.
     */
    void commit();

    /**
     * Called when there is an exception during {@link #accept(Message)} or {@link #commit()}.
     */
    void rollback();


}
