/*
 * (C) Copyright 2016 Nuxeo SA (http://nuxeo.com/) and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package org.nuxeo.ecm.platform.importer.queue.tests;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.nuxeo.ecm.core.test.CoreFeature;
import org.nuxeo.ecm.core.test.annotations.Granularity;
import org.nuxeo.ecm.core.test.annotations.RepositoryConfig;
import org.nuxeo.ecm.platform.importer.log.BufferredLogger;
import org.nuxeo.ecm.platform.importer.log.ImporterLogger;
import org.nuxeo.ecm.platform.importer.queue.manager.CQManager;
import org.nuxeo.ecm.platform.importer.source.SourceNode;
import org.nuxeo.runtime.test.runner.Features;
import org.nuxeo.runtime.test.runner.FeaturesRunner;

import java.util.concurrent.TimeUnit;

import javax.xml.transform.Source;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

@RunWith(FeaturesRunner.class)
@Features(CoreFeature.class)
@RepositoryConfig(cleanup = Granularity.METHOD)
public class TestCQManager {

    protected static final Log log = LogFactory.getLog(TestCQManager.class);

    @Test
    public void readWrite() throws InterruptedException {
        //ImporterLogger logger = mock(ImporterLogger.class);
        // To get logs
        ImporterLogger logger = new BufferredLogger(log);
        SourceNode node = new BuggySourceNode(1, false, false);
        SourceNode node1;
        try (final CQManager qm = new CQManager(logger, 5)) {
            qm.put(1, node);
            qm.put(1, node);
            node1 = qm.poll(1);
        }
        // System.out.println(node1.getName());
        assertEquals(node.getName(), node1.getName());
    }

    @Test
    public void testAutoCloseable() throws Exception {
        ImporterLogger logger = new BufferredLogger(log);
        SourceNode node = new BuggySourceNode(1, false, false);
        CQManager qm = new CQManager(logger, 5);
        qm.put(1, node);
        qm.close();
        try {
            qm.poll(1);
            fail("Not allowed to use a closed queues maanger");
        } catch (IndexOutOfBoundsException e) {
            // expected
        }

        try (final CQManager qm2 = new CQManager(logger, 5, true)) {
            // reopen but at end
            SourceNode node2 = qm2.poll(1, 5, TimeUnit.MILLISECONDS);
            assertEquals(null, node2);
        }
    }

}
